/* NSLU2 control program
   Uses the DTR line on an FTDI USB-serial converter to control the 
   reset line on the NSLU2 
   Written for the UNSW AOS course, 2006 by David Snowdon */ 

#include <stdio.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <termios.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>

int serial_port_fd;

#define DTR_PULSE     2
#define DTR_UP        1
#define DTR_DOWN      0

/* Below partly borrowed from GTKterm */ 
void set_signal(unsigned int param)
{
	int stat_ = 0;
	
	if(serial_port_fd == -1)
		return;
	
	/*if(ioctl(serial_port_fd, TIOCMGET, &stat_) == -1)
		{
			perror("ioctl");
			return;
		}
	*/
	
	if(param == DTR_PULSE){
		stat_ ^= TIOCM_DTR; 
		if(ioctl(serial_port_fd, TIOCMSET, &stat_) == -1)
			perror("ioctl");
		
		usleep(100000); 
		
		stat_ ^= TIOCM_DTR; 
		if(ioctl(serial_port_fd, TIOCMSET, &stat_) == -1)
			perror("ioctl");
	}else{
		/* Command is not DTR_PULSE */ 
		if(param == DTR_UP)
			stat_ &= ~TIOCM_DTR;
		else
			stat_ |= TIOCM_DTR;
		
		if(ioctl(serial_port_fd, TIOCMSET, &stat_) == -1)
			perror("ioctl");
	}
}

void open_port(char *port)
{
	serial_port_fd = open(port, O_RDWR | O_NOCTTY | O_NDELAY);
	if(serial_port_fd == -1){
		fprintf(stderr, "nslu2: error opening serial port\n"); 
		perror("open"); 
	}
}

int main(int argc, char** argv){
	char defport[] = "/dev/ttyUSB0";
	char* port; 
	int i, param; 
	
	port = defport; 
	
	for(i=1; i<argc; i++){
		if(strcasecmp("-p", argv[i]) == 0){
			i++;
			port = argv[i];
		}else if( strcmp("up", argv[i]) == 0){
			param = DTR_UP; 
		}else if(strcmp("down", argv[i]) == 0){
			param = DTR_DOWN; 
		}else if(strcmp("reset", argv[i]) == 0){
			param = DTR_PULSE; 
		}else{
			printf("Usage: nslu2 <-p port> up|down|reset\n"); 
			exit(1);
		}
	}
	
	open_port(port); 
	
	set_signal(param); 
}
